@@ -24,15 +24,16 @@ Log::ProgramInfo - log global info from a perl programs.
 # 0.1.7                 2015-05-20      John Macdonald   Add user-pluggable log routines
 #                                                        Move log parser from test to module.
 # 0.1.8                 2015-06-12      John Macdonald   Catch most signals (not just HUP)
+# 0.1.9                 2016-02-23      John Macdonald   Additions to match paper requests
 #
 
 =head1 VERSION
 
-Version 0.1.8
+Version 0.1.9
 
 =cut
 
-our $VERSION = '0.1.8';
+our $VERSION = '0.1.9';
 
 use feature qw(say);
 use Data::Dumper;
@@ -42,6 +43,8 @@ use DateTime;
 use DateTime::Duration;
 use Carp qw(carp croak cluck);
 use Fcntl qw(:flock);
+use Config;
+use Digest::SHA;
 
 
 =head1 SYNOPSIS
@@ -53,14 +56,15 @@ use Fcntl qw(:flock);
             [ -logdate none|date|time|datetime ]
             [ -stdout                          ]
             [ -suppress                        ]
+            [ -log     $log4perl_log           ]
             );
 
         # main program does lots of stuff...
         exit 0;
 
     After the program has run, this module will automatically
-    log information about this run into a log file.  It will
-    list such things as:
+    log information about this run into a log file and/or to a
+    log object.  It will list such things as:
       - program
         - name
         - version
@@ -71,7 +75,17 @@ use Fcntl qw(:flock);
         - Version
       - run time
 
-    The log is appended to the file:
+    If a -log parameter is provided, it should be a log object that provides
+    an info method (i.e. a Log4Perl object is likely).  The info will be sent
+    sent to this log in addition to writing it to a log file.  This logging
+    is not affected by the -suppress attribute - use that if you don't want a
+    file log written too.
+
+    Warning, the log parser will have to be modified if you need to parse this
+    info out of a log4perl log - there is extra text in the lines (and any
+    other logging from the program) which will need to be pruned.
+
+    The log is appended to the file whose name is determined by:
         LOGDIR/LOGDATE-LOGNAME.LOGEXT
     where
         LOGDIR        defaults to . (the current directory when the program terminates)
@@ -119,6 +133,9 @@ use Fcntl qw(:flock);
         LOGPROGRAMINFO_EXT=EXT     string  sets the target extension
         LOGPROGRAMINFO_DATE=DATE   string  sets the target filename LOGDATE selector
 
+        (there is no environment variable for setting the log attribute, that
+        can only be done within the program)
+
     Adding extra loggable information:
         If you want to add your own classes of loggable info, there are a
         few restrictions.
@@ -137,12 +154,19 @@ use Fcntl qw(:flock);
 
         The $write_entry function passed to my_logger must be called with
         2 or 3 arguments.  The leading arguments are major (and minor if
-        desired) keys, the final one os the value for the key(s).
+        desired) keys, the final one is the value for the key(s).
 
         Try to keep the first key to 7 characters, and the second to 8 to
         keep the log readable by humans.  (It will be parseable even if you
         use longer keys.)
 
+		Help improve the world!  If you are writing additional classes of
+		info loggers, please consider whether they are truly unique to your
+		own environment.  If there is a chance that they would be useful to
+		other environments, please be encouraged to send your logger to be
+		included into Log::ProgramInfo as either a standard default logger
+		or as an available optional logger.
+
     Parsing the log file:
         The log file is designed to be easily parsed.
 
@@ -250,7 +274,7 @@ sub parse1log {
                 push @$list, $val;
             }
             else {
-                die "repeated key: $key" if exists $log->{$key};
+                die "repeated key: {$key} : line {$line}" if exists $log->{$key};
                 $log->{$key} = $val;
             }
         }
@@ -264,7 +288,7 @@ sub parse1log {
                 push @$list, $val;
             }
             else {
-                die "repeated key: $key $key2" if exists $log->{$key}{$key2};
+                die "repeated key: {$key} {$key2}" if exists $log->{$key}{$key2};
                 $log->{$key}{$key2} = $val;
             }
         }
@@ -309,6 +333,7 @@ BEGIN {
         logdate   => "date",
         logname   => $progbase,
         logext    => ".programinfo",
+        log       => undef,
     );
 
     %_omap = (
@@ -372,74 +397,93 @@ sub catch_sig {
     finish_log("Killed with signal: $signame");
 }
 
+my $logfh;
+my $log;
+
 sub log_entry {
-    my $logfh = shift;
+    my $msg;
     if (@_ == 2 ) {
-        printf $logfh "%-7s : %s\n", @_;
+        $msg = sprintf "%-7s : %s", @_;
     } elsif (@_ == 3 ) {
-        printf $logfh "%-7s : %-8s : %s\n", @_;
+        $msg = sprintf "%-7s : %-8s : %s", @_;
     } else {
         my $msg = "log_entry needs 2 or 3 arguments, got "
             . scalar(@_);
         $msg .= ': (' . join( '), (', @_ ) . ')' if @_;
         cluck $msg;
     }
+    _log_entry( $msg );
+}
+
+sub _log_entry {
+    my $msg = shift;
+    say $logfh $msg if $logfh;
+    $log->info( $msg ) if $log;
 }
 
 sub finish_log {
     return if $kill_caught++; # only write log once - first kill, or termination
     my $exit_status = shift;
-    unless ($option{suppress}) {
-        # pull ENV var over-rides
-        while (my ($k, $v) = each %env_options) {
-            $option{$k} = $v;
-        }
-        my $logfh;
-        my $endtime = DateTime->from_epoch(epoch => time);
+	# pull ENV var over-rides
+	while (my ($k, $v) = each %env_options) {
+		$option{$k} = $v;
+	}
+    if (!$option{suppress} || $option{log}) {
+        my $endtime;
+        $log = $option{log};
+        unless ($option{suppress}) {
+            $endtime = DateTime->from_epoch(epoch => time);
 
-        if ($option{stdout}) {
-            open $logfh, ">>&STDOUT";
-        }
-        else {
-            my $dopt = $option{logdate};
-            my $date =
-                ( "none" eq $dopt )   ? ''
-                : ( "date" eq $dopt ) ? $starttime->ymd('')
-                : ( "time" eq $dopt ) ? $starttime->hms('')
-
-                # : ("datetime" eq $dopt) # validated, so must be 'datetime '
-                : ( $starttime->ymd('') . $starttime->hms('') );
-            $date .= '-' if $date;
-            $option{logext} = ".$option{logext}" if $option{logext} =~ m(^[^.]);
-            my $log_path = "$option{logdir}/$date$option{logname}$option{logext}";
-            open( $logfh, ">>", $log_path )
-                or carp "cannot open log file $log_path: $!";
-            say STDERR "Appending log info to $log_path";
-            my $lock_cnt = 0;
-            while (1) {
-                flock $logfh, LOCK_EX and last;
-                croak "$0 [$$]: flock failed on $log_path: $!" if $lock_cnt > 30;
-                say STDERR "Waiting for lock on $log_path" unless $lock_cnt++;
-                print STDERR ".";
-                sleep(2);
+            if ($option{stdout}) {
+                open $logfh, ">>&STDOUT";
             }
-            say "" if $lock_cnt;
-        }
+            else {
+                my $dopt = $option{logdate};
+                my $date =
+                    ( "none" eq $dopt )   ? ''
+                    : ( "date" eq $dopt ) ? $starttime->ymd('')
+                    : ( "time" eq $dopt ) ? $starttime->hms('')
 
-        say $logfh join( ' ', "########", $uid, '(', $gid, ') :', $progbase, @args );
+                    # : ("datetime" eq $dopt) # validated, so must be 'datetime '
+                    : ( $starttime->ymd('') . $starttime->hms('') );
+                $date .= '-' if $date;
+                $option{logext} = ".$option{logext}" if $option{logext} =~ m(^[^.]);
+                my $log_path = "$option{logdir}/$date$option{logname}$option{logext}";
+                open( $logfh, ">>", $log_path )
+                    or carp "cannot open log file $log_path: $!";
+                say STDERR "Appending log info to $log_path";
+                my $lock_cnt = 0;
+                while (1) {
+                    flock $logfh, LOCK_EX and last;
+                    croak "$0 [$$]: flock failed on $log_path: $!" if $lock_cnt > 30;
+                    say STDERR "Waiting for lock on $log_path" unless $lock_cnt++;
+                    print STDERR ".";
+                    sleep(2);
+                }
+                say "" if $lock_cnt;
+                seek $logfh, 2, 0; # make sure we're still at the end now that it is locked
+            }
+
+        }
+        _log_entry( join( ' ', "########", $uid, '(', $gid, ') :', $progbase, @args ) );
 
         my $mod = show_modules();
         for my $key ( sort keys %$mod ) {
             my ( $ver, $loc ) = @{ $mod->{$key} };
-            log_entry( $logfh, MODULE => NAME    => $key );
-            log_entry( $logfh, MODULE => VERSION => $ver );
-            log_entry( $logfh, MODULE => LOC     => $loc );
+            log_entry( MODULE => NAME    => $key );
+            log_entry( MODULE => VERSION => $ver );
+            log_entry( MODULE => LOC     => $loc );
+			if (open my $fd, '<', $loc) {
+				my $sum = Digest::SHA->new(256);
+				$sum->addfile( $fd );
+				log_entry( MODULE => SUM => $sum->hexdigest );
+			}
         }
         for my $inc (@INC) {
-            log_entry( $logfh, INC => $inc );
+            log_entry( INC => $inc );
         }
 
-        log_entry( $logfh, UNAME => $_->[1], do { my $out = qx( uname $_->[0] ); chomp $out; $out } )
+        log_entry( UNAME => $_->[1], do { my $out = qx( uname $_->[0] ); chomp $out; $out } )
             for (
             [ -s => "System" ],
             [ -n => "Name" ],
@@ -447,28 +491,62 @@ sub finish_log {
             [ -v => "OSVer" ],
             [ -m => "Machine" ]
         );
-        log_entry( $logfh, PERL    => $^X );
-        log_entry( $logfh, PERLVer => $] );
-        log_entry( $logfh, User    => $uid );
-        log_entry( $logfh, Group   => $gid );
-        log_entry( $logfh, ProgDir => $FindBin::Bin );
-        log_entry( $logfh, Program => $progbase );
-        log_entry( $logfh, Version => ( $::VERSION // "(No VERSION)" ) );
-        log_entry( $logfh, Args    => scalar(@args) );
+        my $numproc = 0;
+        my $procid  = 'PROC0';
+        if (open my $cpuinfo, '<', '/proc/cpuinfo') {
+            for (<$cpuinfo>) {
+                chomp;
+                next if /^\s*$/;
+                my ($k, $v) = split /\s*:\s*/, $_, 2;
+                if ($k eq 'processor') {
+                    $procid = "PROC$numproc";
+                    ++$numproc;
+                }
+                else {
+                    log_entry( $procid, $k, $v ) if $v =~ /\S/;
+                }
+            }
+            log_entry( PROCs, $numproc );
+        }
+        log_entry( PERL    => $^X );
+        log_entry( PERLVer => $] );
+		if (open my $fd, '<', $^X) {
+			my $sum = Digest::SHA->new(256);
+			$sum->addfile( $fd );
+			log_entry( PERLSUM => $sum->hexdigest );
+		}
+        log_entry( libc    => $Config{libc} );
+		if (open my $fd, '<', $Config{libc}) {
+			my $sum = Digest::SHA->new(256);
+			$sum->addfile( $fd );
+			log_entry( libcSUM => $sum->hexdigest );
+		}
+        log_entry( User    => $uid );
+        log_entry( Group   => $gid );
+        my $progdir = $FindBin::Bin;
+        log_entry( ProgDir => $progdir );
+        log_entry( Program => $progbase );
+        log_entry( Version => ( $::VERSION // "(No VERSION)" ) );
+		if (open my $fd, '<', "$progdir/$progbase") {
+			my $sum = Digest::SHA->new(256);
+			$sum->addfile( $fd );
+			log_entry( ProgSUM => $sum->hexdigest );
+		}
+        log_entry( Args    => scalar(@args) );
         my $acnt = 0;
-        log_entry( $logfh, "  arg" => sprintf("%8d", ++$acnt), $args[$acnt-1] ) for @args;
-        log_entry( $logfh, Start   => $starttime->datetime() . "." . sprintf( "%03d", $starttime->millisecond ) );
-        log_entry( $logfh, End     => $endtime->datetime()   . "." . sprintf( "%03d", $endtime->millisecond ) );
+        log_entry( "  arg" => sprintf("%8d", ++$acnt), $args[$acnt-1] ) for @args;
+        log_entry( Start   => $starttime->datetime() . "." . sprintf( "%03d", $starttime->millisecond ) );
+        log_entry( End     => $endtime->datetime()   . "." . sprintf( "%03d", $endtime->millisecond ) );
         my $dur = $endtime->subtract_datetime_absolute($starttime);
-        log_entry( $logfh, Elapsed => $dur->delta_seconds . "." .
+        log_entry( Elapsed => $dur->delta_seconds . "." .
                                     sprintf( "%03d", $dur->delta_nanoseconds/1_000_000) );
-        log_entry( $logfh, EndStat => $exit_status );
+        log_entry( EndStat => $exit_status );
 
-        $_->(sub { log_entry( $logfh, @_ ) }) for @extra_loggers;
+        $_->(sub { log_entry( @_ ) }) for @extra_loggers;
 
-        say $logfh "";
+        _log_entry ""; # blank line to separate any appended later log
 
-        close($logfh);
+        close $logfh if $logfh;
     }
 }
 
